function randomEwE = f_EwEnormal_MonteCarlo_04092014(EwEResult, MonteCarlo_interval, PEDIGREE)
% generate 1 random EwE model from a NORMAL distribution_type - NOT necessarily balanced
% takes:
%       PEDIGREE
%       EwEResult
%       MonteCarlo_interval
% returns:
%       randomEwE           (1 random EwE model - NOT necessarily balanced)
% calls:
%       f_DietPreference_Readjust_04052014
%       f_EE_MonteCarlo_04092014
% revision date: 4-9-2014


% *************************************************************************
% STEP 1: unpack-----------------------------------------------------------
Original_Parameters     = MonteCarlo_interval.Original_Parameters; % [EwE_PB EwE_QB EwE_PQ EwE_AE EwE_MTBLSM EwE_Biomass];
PEDIGREE_Parameters_STD = MonteCarlo_interval.PEDIGREE_Parameters_STD; % PEDIGREE_Parameters matrix as STD values (CV * mean = STD)
low_PARAMinterval       = MonteCarlo_interval.low_PARAMinterval;
high_PARAMinterval      = MonteCarlo_interval.high_PARAMinterval;
Params_rows             = MonteCarlo_interval.Params_rows;
Params_clms             = MonteCarlo_interval.Params_clms;

Original_Landings       = MonteCarlo_interval.Original_Landings;
PEDIGREE_Landings_STD   = MonteCarlo_interval.PEDIGREE_Landings_STD; % PEDIGREE_Landings matrix as STD values (CV * mean = STD)
low_LANDINGSinterval    = MonteCarlo_interval.low_LANDINGSinterval;
high_LANDINGSinterval   = MonteCarlo_interval.high_LANDINGSinterval;
Landings_rows           = MonteCarlo_interval.Landings_rows;
Landings_clms           = MonteCarlo_interval.Landings_clms;

Original_Discards       = MonteCarlo_interval.Original_Discards;
PEDIGREE_Discards_STD   = MonteCarlo_interval.PEDIGREE_Discards_STD; % PEDIGREE_Discards matrix as STD values (CV * mean = STD)
low_DISCARDSinterval    = MonteCarlo_interval.low_DISCARDSinterval;
high_DISCARDSinterval   = MonteCarlo_interval.high_DISCARDSinterval;

Original_Diet           = MonteCarlo_interval.Original_Diet;
PEDIGREE_Diet_STD       = MonteCarlo_interval.PEDIGREE_Diet_STD;
low_DIETinterval        = MonteCarlo_interval.low_DIETinterval;
high_DIETinterval       = MonteCarlo_interval.high_DIETinterval;
Diet_rows               = MonteCarlo_interval.Diet_rows;
Diet_clms               = MonteCarlo_interval.Diet_clms;

Original_BAEM           = MonteCarlo_interval.Original_BAEM;
PEDIGREE_BAEM_STD       = MonteCarlo_interval.PEDIGREE_BAEM_STD;
low_BAEMinterval        = MonteCarlo_interval.low_BAEMinterval;
high_BAEMinterval       = MonteCarlo_interval.high_BAEMinterval;
BAEM_rows               = MonteCarlo_interval.BAEM_rows;

looky_detritus          = MonteCarlo_interval.looky_detritus; % includes eggs
looky_ANYPrimaryProd    = MonteCarlo_interval.looky_ANYPrimaryProd;
STD_scaler              = MonteCarlo_interval.STD_scaler;
num_EwEgroups           = MonteCarlo_interval.num_EwEgroups;


GroupType                = EwEResult.GroupType;
EwE_Diet                 = EwEResult.Diet;
num_detritus             = EwEResult.num_detritus;
num_fisheries            = EwEResult.num_fisheries;
num_EggsAndDetritus      = EwEResult.num_EggsAndDetritus;

EwE_DetritusFate         = EwEResult.EwE_DetritusFate;
GroupTypeDef_ANYDetritus = EwEResult.GroupTypeDef_ANYDetritus;
GroupTypeDef_eggs        = EwEResult.GroupTypeDef_eggs;


% *************************************************************************
% STEP 2: generate random model--------------------------------------------
% step 2a: PARAMETERS (clm order: [PB QB PQ AE MTBLSM B]) -----------------
FreeParams                  = [1 3 4 6]; % parameters that vary freely [PB PQ AE Biomass]
MonteCarlo_ParametersFree   = zeros(Params_rows, 4); % initialize
Original_ParametersFree     = Original_Parameters(:, FreeParams);
PEDIGREE_Parameters_STDFree = PEDIGREE_Parameters_STD(:, FreeParams);
low_PARAMintervalFree       = low_PARAMinterval(:, FreeParams);
high_PARAMintervalFree      = high_PARAMinterval(:, FreeParams);
looky_redo                  = 1:(Params_rows * 4); % redo addresses of free parameter matrix are merged into 1 vector
looky_redo                  = looky_redo';
num_redo                    = length(looky_redo); % number of param to redraw
loop_counter                = 0;

while num_redo > 0 && loop_counter <= 1000
    
    % draw random parameter set
    MonteCarlo_ParametersFree(looky_redo) = Original_ParametersFree(looky_redo) + (PEDIGREE_Parameters_STDFree(looky_redo) .* randn(num_redo, 1)); % normal distribution

    % evaluate parameter set (evaluate only the FREE parameters)
    looky_TooLow                      = find((MonteCarlo_ParametersFree - low_PARAMintervalFree)     < 0);
    looky_TooHigh                     = find((high_PARAMintervalFree    - MonteCarlo_ParametersFree) < 0);
    looky_redo                        = unique([looky_TooLow; looky_TooHigh]);
    num_redo                          = length(looky_redo); % number of param to redraw
    loop_counter                      = loop_counter + 1;
    
end % (while num_redo > 0 && loop_counter <= 1000)

% evaluate physiology parameters & replace any remaining bad rows with original parameter sets
MonteCarlo_ParametersPhysio                     = MonteCarlo_ParametersFree(:, 1:3); % [PB PQ AE]
Original_ParametersPhysio                       = Original_ParametersFree(:, 1:3); % [PB PQ AE]
low_PARAMintervalPhysio                         = low_PARAMintervalFree(:, 1:3); % [PB PQ AE]
high_PARAMintervalPhysio                        = high_PARAMintervalFree(:, 1:3); % [PB PQ AE]
[looky_TooLow_rows,  looky_TooLow_clms]         = find((MonteCarlo_ParametersPhysio - low_PARAMintervalPhysio)   < 0);
[looky_TooHigh_rows, looky_TooHigh_clms]        = find((high_PARAMintervalPhysio  - MonteCarlo_ParametersPhysio) < 0);
looky_redo_rows                                 = unique([looky_TooLow_rows; looky_TooHigh_rows]);
% looky_redo_clms                                 = unique([looky_TooLow_clms; looky_TooHigh_clms]);
MonteCarlo_ParametersPhysio(looky_redo_rows, :) = Original_ParametersPhysio(looky_redo_rows, :); % replace the whole row
MonteCarlo_ParametersFree(looky_redo_rows, 1:3) = MonteCarlo_ParametersPhysio(looky_redo_rows, :);

% evaluate biomass & replace any remaining bad rows with original biomass
MonteCarlo_ParametersBiomass                    = MonteCarlo_ParametersFree(:, 4); % [B]
Original_ParametersBiomass                      = Original_ParametersFree(:, 4); % [B]
low_PARAMintervalBiomass                        = low_PARAMintervalFree(:, 4); % [B]
high_PARAMintervalBiomass                       = high_PARAMintervalFree(:, 4); % [B]
looky_TooLow                                    = find((MonteCarlo_ParametersBiomass - low_PARAMintervalBiomass)     < 0);
looky_TooHigh                                   = find((high_PARAMintervalBiomass    - MonteCarlo_ParametersBiomass) < 0);
looky_redo                                      = unique([looky_TooLow; looky_TooHigh]);
MonteCarlo_ParametersBiomass(looky_redo, :)     = Original_ParametersBiomass(looky_redo, :); % replace the biomass
MonteCarlo_ParametersFree(looky_redo, 4)        = MonteCarlo_ParametersBiomass(looky_redo, :);

% re-calculate non-free parameters (QB & MTBLSM)
PB                    = MonteCarlo_ParametersFree(:, 1);
PQ                    = MonteCarlo_ParametersFree(:, 2);
AE                    = MonteCarlo_ParametersFree(:, 3);
Biomass               = MonteCarlo_ParametersFree(:, 4);
QB                    = PB ./ PQ; % recalculate QB = PB / PQ; % NOTE: make a choice -- you can either calculate QB or calculate PB;
QB(isnan(QB))         = 0; % filter out div/0 errors; (vertical vector)
MTBLSM                = AE - PQ; % recalculate MTBLSM = AE - PQ
MonteCarlo_Parameters = [PB QB PQ AE MTBLSM Biomass]; % put together final MonteCarlo_Parameters matrix

% step 2b: Landings -------------------------------------------------------
MonteCarlo_Landings = Original_Landings + (PEDIGREE_Landings_STD .* randn(Landings_rows, Landings_clms)); % normal distribution

% filter out out-of-range values and reset them to original values
% all values are kept within [low_LANDINGSinterval high_LANDINGSinterval] but still drawn from normal distribution
looky_TooLow                                       = find((MonteCarlo_Landings   - low_LANDINGSinterval) < 0);
looky_TooHigh                                      = find((high_LANDINGSinterval - MonteCarlo_Landings)  < 0);
MonteCarlo_Landings([looky_TooLow; looky_TooHigh]) = Original_Landings([looky_TooLow; looky_TooHigh]);

% step 2c: Discards -------------------------------------------------------
MonteCarlo_Discards = Original_Discards + (PEDIGREE_Discards_STD .* randn(Landings_rows, Landings_clms)); % normal distribution

% filter out out-of-range values and reset them to original values
% all values are kept within [low_DISCARDSinterval high_DISCARDSinterval] but still drawn from normal distribution
looky_TooLow                                       = find((MonteCarlo_Discards   - low_DISCARDSinterval) < 0);
looky_TooHigh                                      = find((high_DISCARDSinterval - MonteCarlo_Discards)  < 0);
MonteCarlo_Discards([looky_TooLow; looky_TooHigh]) = Original_Discards([looky_TooLow; looky_TooHigh]);
        
% step 2d: DIET -----------------------------------------------------------
MonteCarlo_Diet = Original_Diet + (PEDIGREE_Diet_STD .* randn(Diet_rows, Diet_clms)); % normal distribution
    
% filter out out-of-range values and reset them to original values
% all values are kept within [low_DIETinterval high_DIETinterval] but still drawn from normal distribution
looky_TooLow                                   = find((MonteCarlo_Diet   - low_DIETinterval) < 0);
looky_TooHigh                                  = find((high_DIETinterval - MonteCarlo_Diet)  < 0);
MonteCarlo_Diet([looky_TooLow; looky_TooHigh]) = Original_Diet([looky_TooLow; looky_TooHigh]);

% % no negative diet entries
% MonteCarlo_Diet(MonteCarlo_Diet < 0) = 0; % the interval should weed out all diet entries < 0 or > 1

% step 2e: BA & EM --------------------------------------------------------
MonteCarlo_BAEM = Original_BAEM + (PEDIGREE_BAEM_STD .* randn(BAEM_rows, 2)); % normal distribution

% filter out out-of-range values and reset them to original values
% all values are kept within [low_DIETinterval high_DIETinterval] but still drawn from normal distribution
looky_TooLow                                   = find((MonteCarlo_BAEM   - low_BAEMinterval) < 0);
looky_TooHigh                                  = find((high_BAEMinterval - MonteCarlo_BAEM)  < 0);
MonteCarlo_BAEM([looky_TooLow; looky_TooHigh]) = Original_BAEM([looky_TooLow; looky_TooHigh]);





% *************************************************************************
% STEP 3: finalize and pack MonteCarlo parameters for exporting------------
% step 3a: parameters [PB QB PQ AE MTBLSM B] ------------------------------
randomEwE.PB                      = MonteCarlo_Parameters(:, 1); % (vertical vector)
randomEwE.QB                      = MonteCarlo_Parameters(:, 2); % (vertical vector)
randomEwE.PQ                      = MonteCarlo_Parameters(:, 3); % (vertical vector)
randomEwE.AE                      = MonteCarlo_Parameters(:, 4); % (vertical vector)
randomEwE.MTBLSM                  = MonteCarlo_Parameters(:, 5); % (vertical vector)
randomEwE.Biomass                 = MonteCarlo_Parameters(:, 6); % (vertical vector)
randomEwE.BA                      = MonteCarlo_BAEM(:, 1); % (vertical vector)
randomEwE.EM                      = MonteCarlo_BAEM(:, 2); % (vertical vector)
randomEwE.Production              = randomEwE.Biomass .* randomEwE.PB; % (vertical vector); (NOTE: local production only, does not include immigration)
randomEwE.TotalConsumption        = randomEwE.Biomass .* randomEwE.QB; % total consumption (t/km2/y); (vertical vector)

% step 3b: fisheries ------------------------------------------------------
randomEwE.Landings                = MonteCarlo_Landings; % (array of vertical vectors)
randomEwE.Discards                = MonteCarlo_Discards; % (array of vertical vectors)
randomEwE.TotalFisheries          = MonteCarlo_Landings + MonteCarlo_Discards; % (array of vertical vectors)
randomEwE.Biomass_fisheries       = sum(randomEwE.TotalFisheries, 1)'; % (1 vertical vector)
randomEwE.Production_fisheries    = sum(MonteCarlo_Landings, 1)'; % (1 vertical vector)
randomEwE.PB_fisheries            = (randomEwE.Production_fisheries ./ randomEwE.Biomass_fisheries); % (1 vertical vector)
randomEwE.QB_fisheries            = (randomEwE.Biomass_fisheries    ./ randomEwE.Biomass_fisheries); % (1 vertical vector)
looky_NaN                         = find(randomEwE.Biomass_fisheries <=0);
randomEwE.PB_fisheries(looky_NaN) = 0;
randomEwE.QB_fisheries(looky_NaN) = 0;
randomEwE.PQ_fisheries            = randomEwE.PB_fisheries; % (1 vertical vector)
randomEwE.AE_fisheries            = randomEwE.PB_fisheries; % (1 vertical vector)
randomEwE.BA_fisheries            = zeros(Landings_clms, 1); % BA for fisheries is always 0; (1 vertical vector)
randomEwE.EM_fisheries            = randomEwE.Production_fisheries; % (1 vertical vector)

% step 3c: calculate bioenergetic budget for each producer ----------------
%       p = consumption budget = the fraction of consumption (Q) going to production + feces (non-assimilated consumption) + metabolism
%       (NOTE: internal BioenergeticBudget, does not include EM)
BioenergeticBudget_production     = randomEwE.PQ;
BioenergeticBudget_feces          = (1 - randomEwE.AE);
BioenergeticBudget_metabolism     = randomEwE.MTBLSM; % fraction of consumption going to metabolism (NH4 production); Metabolism = 1 - PQ - feces = (1 - (EwE_PQ + (1-EwE_AE)))

% step 3d: diet matrix & consumption matrix -------------------------------
% QQQ is MonteCarlo_Diet normalized to 1 at this point??

% adjust diet - re-distribute diet preferences within functional group clusters based on new randomEwE production rates
MonteCarlo_Diet = f_DietPreference_Readjust_04052014(PEDIGREE, MonteCarlo_Diet, randomEwE.Production);
sum_Diet        = sum(MonteCarlo_Diet, 1); % (horizontal vector)
MonteCarlo_Diet = MonteCarlo_Diet ./ repmat(sum_Diet, [Diet_rows, 1]);
MonteCarlo_Diet(:, [looky_ANYPrimaryProd; looky_detritus]) = 0; % correct for div/0 NaNs (primary producers & detritus)
Consumption     = MonteCarlo_Diet .* repmat(randomEwE.TotalConsumption', [Diet_rows, 1]); % (2D matrix: (groups + import row) X groups); NOTE transpose of randomEwE.TotalConsumption

randomEwE.Diet                          = EwE_Diet; % initialize with header row & clm; (HAS import row)
randomEwE.Diet(2:end, 2:end)            = MonteCarlo_Diet;
randomEwE.Consumption                   = Consumption; % NOTE: does NOT include header row & clm
% NOTE: this leaves consumption by detritus groups = 0
% NOTE: does not include fisheries (because this is EwE model)

% step 3e: calculate total predation & fishing pressure on each group -----
TotalPredation                       = sum(randomEwE.Consumption(1:(end-1), :), 2); % total predation  on each group (trimming off import consumption row); (vertical vector)
TotalPredation                       = TotalPredation + sum(randomEwE.TotalFisheries, 2); % add Landings + Discards to TotalPredation; (vertical vector)

% step 3f: calculate detritus production & consumption --------------------
production_feces                     = BioenergeticBudget_feces .* randomEwE.TotalConsumption; % (vertical vector); NOTE: detritus production terms here are a function of random rTotalConsumption = Q/B * Biomass parameters and therefore need to be replaced
production_OtherMortality            = (randomEwE.Production - randomEwE.EM) - TotalPredation; % (vertical vector); NOTE: detritus production terms (randomEwE.Production) here are a function of random P = PB * Biomass parameters and therefore need to be replaced; NOTE: subtraction of EM from production because a negative EM = immigration = addition to production

% QQQ correct to do this???
looky_neg                            = find(production_OtherMortality < 0);
production_OtherMortality(looky_neg) = 0; % do not allow negative detritus production (over-use of detritus will show up in EE metric)

production_detritus                  = production_feces + production_OtherMortality; % total detritus production, calculated from random AE value and Consumption matrix; (vertical vector)
production_detritus                  = [production_detritus; sum(randomEwE.Discards, 1)']; % append fishery discards as rows to bottom of EwE groups; (vertical vector)
production_detritus(looky_detritus)  = 0; % set original detritus production to 0 (we are re-calculating this and don't want old value that was calculated with detritus Q/B values)

% apply detritus fate distribution (use EwE fates since this is an EwE model)
%       NOTE: this works ONLY for 1-step detritus-to-detritus links
detritus_to_detrius_fate             = EwE_DetritusFate(looky_detritus, :); % (2D matrix: num_detritus X num_detritus)
TotalPredation_detritus              = TotalPredation(looky_detritus); % total predation UPON each detritus group (vertical vector)
build_detritus                       = repmat(production_detritus, [1, num_EggsAndDetritus]) .* EwE_DetritusFate; % (2D matrix: groups (w/ fisheries) X num_detritus); NOTE: detritus rows are 0
sum_build_detritus                   = sum(build_detritus, 1); % total flow into each detritus group; (horizontal vector: 1 X num_EggsAndDetritus)
SurplusProduction_detritus           = sum_build_detritus' - TotalPredation_detritus; % total UNconsumed detritus; (vertical vector: num_detritus X 1)
detritus_to_detritus                 = repmat(SurplusProduction_detritus, [1, num_EggsAndDetritus]) .* detritus_to_detrius_fate; % flow between detritus boxes; (2D matrix: num_EggsAndDetritus X num_EggsAndDetritus)

% correct detritus entries
looky_neg                                        = find(detritus_to_detritus < 0);
detritus_to_detritus(looky_neg)                  = 0; % do not allow negative detritus production (over-use of detritus will show up in EE metric)
build_detritus(looky_detritus, :)                = detritus_to_detritus; % put detritus flow back into build_detritus (set of vertical vectors)
randomEwE.Consumption(1:(end-1), looky_detritus) = build_detritus(1:(end-num_fisheries), :); % put detritus flow into Consumption matrix (NOT including fishery discard contribution in build_detritus, not writing over import consumtion in Consumption); last row still import consumption
randomEwE.TotalConsumption(looky_detritus)       = sum(build_detritus, 1)'; % replace flow into detritus boxes in TotalConsumption vector; DOES sum in discard detritus; (vertical vector)
randomEwE.Production(looky_detritus)             = sum(build_detritus, 1)'; % replace flow into detritus boxes in Production vector; DOES sum in discard detritus; (vertical vector)
randomEwE.PB(looky_detritus)                     = randomEwE.Production(looky_detritus) ./ randomEwE.Biomass(looky_detritus); % NOTE: PB value for detritus is a function of MonteCarlo biomass, meaning is therefore somewhat arbitrary
randomEwE.QB(looky_detritus)                     = randomEwE.PB(looky_detritus); % NOTE: QB value for detritus is a function of MonteCarlo biomass, meaning is therefore somewhat arbitrary
temp_predation                                   = sum(randomEwE.Consumption, 2); % total predation INCLUDING flow to detritus
randomEwE.EM(looky_detritus)                     = (randomEwE.Production(looky_detritus) - temp_predation(looky_detritus)); % (might not ever use this, but including to be consistent with ECOTRAN)

% % step 3g: predation matrix & mortality rates ---------------------------
% randomEwE.Predation = zeros((Diet_rows-1), Diet_clms); % initialize Predation matrix (this EXCLUDES import consumption row)
% for predator_loop = 1:Diet_clms
%     randomEwE.Predation(:, predator_loop) = randomEwE.Consumption(1:(end-1), predator_loop) ./ randomEwE.Biomass; % (NO import consumption row); (NO headers)
% end
% randomEwE.M2total          = nansum(randomEwE.Predation, 2); % total predation mortality (1/y); (vertical vector); (NO import consumption row)
% randomEwE.FisheryMortality = sum(randomEwE.TotalFisheries, 2) ./ randomEwE.Biomass; % total fishery mortality (1/y); (vertical vector)

% step 3h: add GroupType to randomEwE -------------------------------------
randomEwE.GroupType                = GroupType;
randomEwE.GroupTypeDef_ANYDetritus = GroupTypeDef_ANYDetritus;
randomEwE.GroupTypeDef_eggs        = GroupTypeDef_eggs;
randomEwE.looky_detritus           = looky_detritus; % includes eggs

randomEwE.num_fisheries            = num_fisheries;
randomEwE.STD_scaler               = STD_scaler; % retain this here to recall scaler outside of m-file

% step 3i: calculate EcotrophicEfficiency ---------------------------------
randomEwE                          = f_EE_MonteCarlo_04092014(randomEwE);
% randomEwE.EcotrophicEfficiency(looky_detritus) = 0; % QQQ don't evaluate based on eggs or detritus


% end m-file***************************************************************