% ECOTRAN_StaticScenarios_TEST_08052018
% run and plot static scenarios
% by Jim Ruzicka
% calls:
%           f_read_EwE_csv_04292016                     read in EwE model as variable 'dat'
%           f_AggregateResults_EwE_03122015             prepare EwE model for use by ECOTRAN; also, aggregate functional groups here if wanted
%           ECOTRANuncertainty_05062016
%                   f_ECOfunction_05142015              returns a single ECOTRAN model for 1 "type" EwE model or 1 MonteCarlo EwE model
%                   f_RedistributeCannibalism           remove cannibalism terms on diagonal of matrix EwE_diet
%                   f_CalcPredationMatrix               for each producer, p, and consumer, c: ((b_pc * Q_c) / M2_p)) = the fraction of total predation on each producer, p, going to each consumer, c
%           f_E2E_pedigree_08032018                     generate the pedigree (CV) of each cell of the EnergyBudget_matrix (A_cp)
%           f_E2E_MonteCarlo_08032018                   generate Monte Carlo models from the EnergyBudget_matrix (A_cp)
%           f_WebProductivityWLoss                      calculate production rates of all groups under a given driver (e.g., nitrate or primary production); also accounts for defined rates of group production export when running under static scenarios
%           f_ScenarioGenerator_08302013                modify the EnergyBudget_matrix according to forced scenario conditions
%                   f_WebProductivityWLoss
%           f_CompileScenarioResults_08192013
%           p_PlotScenarioResults_02092018
%                   myboxplot_4 & myboxutil_2           (from the Jorn_Diedrichsen_Toolbox; found at www.icn.ucl.ac.uk/motorcontrol/toolboxes/matlab_toolboxes.htm; myboxplot_4 and myboxutil_2 are modified versions; other boxplot functions could be used instead)
%                       
% revision date: 8-5-2018


% *************************************************************************
% STEP 1: load & aggregate EwE results-------------------------------------
% step 1a: define model to work with
ReadFile_directory         = '/Users/ruzickaj/Documents/ECOTRAN_08052018/FoodWeb_models/TEST_model/';
readFile                   = 'TESTmodel_07252018.csv';
readFile                   = [ReadFile_directory readFile];

% step 1b: load ECOPATH (EwE) model from Aydin VisualBasic file (.csv format)
dat                  	= f_read_EwE_csv_04292016(readFile); % use for models with 10 primary producers

% step 1c: aggregate model results & prep EwEResult for analysis ----------
[EwEResult, PEDIGREE] 	= f_AggregateResults_EwE_03122015(dat);





% *************************************************************************
% STEP 2: load pre-generated Monte Carlo EwE models------------------------
% step 2a: Identify Monte Carlo method ------------------------------------
%          method 1: PREFERRED METHOD --> start with the one original ECOTRAN base model and generate a set of Monte Carlo E2E models of the ECOTRAN production matrix using a predefined CV (or defualt pedigree CV = 0.5)
%          method 2: use pre-generated Monte Carlo EwE models, convert to ECOTRAN E2E models, and run scenarios directly on EACH of these individually
%                    (each matrix being its own base model and scenario model)
MonteCarlo_method           = 1;    % SSS; choose 1 or 2; NOTE: method 2 is no longer supported
use_MonteCarlo              = 'y';  % SSS; use MonteCarlo uncertainty estimates?

% step 2b: if method 2 is chosen, load MonteCarloSet ----------------------
if strcmp(use_MonteCarlo, 'y') && MonteCarlo_method == 2
    
    % define MonteCarloSet  = name of Monte Carlo results with file directory (loads 'MonteCarloStore')
    MonteCarloSet_FileDirectory = '/Users/ruzickaj/Documents/ECOTRAN_08052018/FoodWeb_models/TEST_model/';   % NOTE laptop MAC directory format
    MonteCarloSet               = 'TEST_MCnorm_18-Jan-2015.mat';   % sampling interval = +/1 1 STD sampling (1 std in denominator)
    MonteCarloSet               = [MonteCarloSet_FileDirectory MonteCarloSet];
    
    load(MonteCarloSet, 'MonteCarloStore')
else
    MonteCarloStore          = [];
end





% *************************************************************************
% STEP 3: ECOTRAN conversion-----------------------------------------------
% step 3a: perform ECOTRAN conversion -------------------------------------
[ECOTRAN]                                       = ECOTRANuncertainty_05062016(EwEResult, MonteCarloStore);

% step 3b: read in ECOTRAN structure variables ----------------------------
%          (so that no changes made to original values)
GroupType                                        = ECOTRAN.GroupType;
AggLabel                                         = ECOTRAN.AggLabel;
EnergyBudget_matrix                              = ECOTRAN.EnergyBudget_matrix;
EwE_Biomass                                      = ECOTRAN.EwE_Biomass;
EwE_PB                                           = ECOTRAN.EwE_PB;
EwE_QB                                           = ECOTRAN.EwE_QB;
DetritusFate_feces                               = ECOTRAN.DetritusFate_feces;
DetritusFate_senescence                          = ECOTRAN.DetritusFate_senescence;
ExcretionFate                                    = ECOTRAN.ExcretionFate;
EggFate                                          = ECOTRAN.EggFate;
ProductionLossScaler                             = ECOTRAN.ProductionLossScaler;
RetentionScaler                                  = ECOTRAN.RetentionScaler;     % sensitivity to advection & mixing (0 = more advection <--> less advection =1); (horizontal vector; without nutrient clms)
FunctionalResponseParams                         = ECOTRAN.FunctionalResponseParams;
[num_E2Egroups, E2E_clms, E2E_layers]            = size(EnergyBudget_matrix);
% TransferEfficiency                               = ECOTRAN.TransferEfficiency; % gets redefined manually below

% step 3c: find detritus, nutrients, BA & EM ------------------------------
%          by row address in EnergyBudget_matrix
looky_NUTRIENTS                                  = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYNitroNutr);
looky_NO3                                        = find(GroupType        == ECOTRAN.GroupTypeDef_NO3);
looky_plgcNH4                                    = find(GroupType        == ECOTRAN.GroupTypeDef_plgcNH4);
looky_bnthNH4                                    = find(GroupType        == ECOTRAN.GroupTypeDef_bnthNH4);
looky_PrimaryProducer                            = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYPrimaryProd);
looky_Macroalgae                                 = find(GroupType        == ECOTRAN.GroupTypeDef_Macrophytes);
looky_Phytoplankton                              = looky_PrimaryProducer(~ismember(looky_PrimaryProducer, looky_Macroalgae));
looky_DETRITUS                                   = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYDetritus);
looky_terminalPlgcDetr                           = find(GroupType        == ECOTRAN.GroupTypeDef_terminalPlgcDetr);
looky_terminalBnthDetr                           = find(GroupType        == ECOTRAN.GroupTypeDef_terminalBnthDetr);
looky_eggs                                       = find(GroupType        == ECOTRAN.GroupTypeDef_eggs);
looky_ANYConsumer                                = find(floor(GroupType) == ECOTRAN.GroupTypeDef_ANYConsumer);
looky_fishery                                    = find(GroupType        == ECOTRAN.GroupTypeDef_fishery);
looky_nonNO3                                     = 1:E2E_clms;
looky_nonNO3(looky_NO3)                          = [];
% looky_nonNO3(looky_NUTRIENTS)                    = [];                        % SSS add NH4 if NH4 is also a physically imported driver
num_Nutrients                                    = length(looky_NUTRIENTS);
num_NH4                                          = length([looky_plgcNH4; looky_bnthNH4]);
num_PrimaryProducers                             = length(looky_PrimaryProducer);
num_Macroalgae                                   = length(looky_Macroalgae);
num_Phytoplankton                                = length(looky_Phytoplankton);
num_eggs                                         = length(looky_eggs);
num_detritus                                     = length(looky_DETRITUS);

% step 3d: matrix addresses of TESTmodel_07252018 groups (for easier plotting)
%          NOTE: these must be entered manually for each new model
NO3_rowclm          = 1;
PlgcNH4_rowclm      = 2;
BnthNH4_rowclm      = 3;
SmlPhyto_rowclm     = 4;
LargePhyto_rowclm   = 5;
MicroZ_rowclm       = 6;
MesoZ_rowclm        = 7;
MacroZ_rowclm       = 8;
HerbiJelly_rowclm   = 9;
CarniJelly_rowclm   = 10;
Euphausid_rowclm    = 11;
Cephalo_rowclm      = 12;
PlankFish_rowclm	= 13;
PiscFish_rowclm     = 14;
BenthFish_rowclm	= 15;
JuvFish_rowclm      = 16;
DepFBen_rowclm      = 17;
CarnBen_rowclm      = 18;
SuspFBen_rowclm     = 19;
SeaBird_rowclm      = 20;
GrayWhale_rowclm    = 21;
BalWhal_rowclm      = 22;
Pinniped_rowclm     = 23;
Odontocete_rowclm   = 24;
FishEggs_rowclm     = 25;
InvertEggs_rowclm   = 26;
Fishery1_rowclm     = 27; % <<--note that fisheries come before terminal detritus
Fishery2_rowclm     = 28; % <<--note that fisheries come before terminal detritus
PlgcDetr_rowclm     = 29;
BnthcDetr_rowclm	= 30;





% STEP 4: Generate E2E Monte Carlo models based on ECOTRAN EnergyBudget_matrix--------------------------------------------
%         NOTE: this is Monte Carlo method 1
%         method 1: start with the one original ECOTRAN base model and generate a set of Monte Carlo E2E models of the ECOTRAN production matrix using a predefined CV (or defualt pedigree CV = 0.5)
num_webMonteCarlo                                           = 1000; % SSS set this value
if MonteCarlo_method == 1
    
    PEDIGREE.PEDIGREE_eggs                                  = 0.1; % SSS egg pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_predation                             = 0.1; % SSS predation pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_senescence                            = 0.1; % SSS senescence pedigree for all groups (CV)
    PEDIGREE.PEDIGREE_BacterialMTBLSM                       = 0.1; % SSS pedigree for implicit bacterial metabolism of terminal detritus (CV)
    PEDIGREE.PEDIGREE_NutrientUptake                        = 0.1; % SSS pedigree for nutrient uptake by primary producers (CV)
    ECOTRAN_PEDIGREE                                        = f_E2E_pedigree_08032018(EwEResult, PEDIGREE, ECOTRAN);
    
            % SSS use for standardized pedigree
            [rows, clms]                                = size(ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix);
            ECOTRAN_PEDIGREE.CV_EnergyBudget_matrix     = 0.75 * ones(rows, clms);
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget       = zeros(7, clms);
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(1, :) = 0.25; % feces
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(2, :) = 0.25; % metabolism
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(3, :) = 0.25; % eggs
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(4, :) = 0.1; % predation
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(5, :) = 0.1; % senescence
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(6, :) = 0; % BA
            ECOTRAN_PEDIGREE.CV_ConsumptionBudget(7, :) = 0; % EM
    
    MonteCarloConditions.num_trials                         = num_webMonteCarlo;            % number of trials to run
    MonteCarloConditions.EnergyBudget_matrix                = ECOTRAN.EnergyBudget_matrix;
    MonteCarloConditions.DistributionType                   = 'normal';                     % 'normal' or 'uniform'
    [ConsumptionBudget_MonteCarlo, EnergyBudget_MonteCarlo] = f_E2E_MonteCarlo_08032018(MonteCarloConditions, ECOTRAN_PEDIGREE);

    EnergyBudget_matrix                                     = EnergyBudget_MonteCarlo;
    E2E_layers                                              = MonteCarloConditions.num_trials;

end % (choose MonteCarlo_method)





% *************************************************************************
% STEP 5: TransferEfficiency-----------------------------------------------
% SET MANUALLY: TransferEfficiency = 1 for all groups because we are
%           working with the consumption budget matrix that tracks the fate of ALL consumption (not just predation)
%           system losses (groups where TransferEfficiency < 1 include benthic detritus, fishery, (others?)
TransferEfficiency                              = ones(E2E_layers, E2E_clms);	% re-initialize as rows of horizontal vector of 1s
TransferEfficiency(:, looky_terminalBnthDetr)   = 0.1;                          % SSS set default TE for benthic detritus





% *************************************************************************
% STEP 6: define production loss fractions---------------------------------
% (FFF in future, might use vector read in from .csv file; but for now, this is all handled here)
% LX                   = 0; % NOTE: set to 0 to ignore physical losses; set to 1 for the default relative upwelling index (1 is used for the average yearly relative upwelling index)
LX_base                       = ones(1, E2E_clms);
ProductionLossScaler_base     = zeros(1, E2E_clms); % initialize
PhysicalLossFraction_base     = ProductionLossScaler_base .* LX_base;





% *************************************************************************
% STEP 7: initialize InputProductionVector---------------------------------
%         NOTE: here you can run a scenario with an alternate driver
InputProductionVector_base                          = zeros(1, E2E_clms); % initialize InputProductionVector_base
InputProductionVector_scenario                      = zeros(1, E2E_clms); % initialize InputProductionVector_scenario

P_SmlPhyto_base                                     = ECOTRAN.EwE_PB(SmlPhyto_rowclm)   * ECOTRAN.EwE_Biomass(SmlPhyto_rowclm);
P_LargePhyto_base                                   = ECOTRAN.EwE_PB(LargePhyto_rowclm) * ECOTRAN.EwE_Biomass(LargePhyto_rowclm);

InputProductionVector_base(SmlPhyto_rowclm)       	= P_SmlPhyto_base;
InputProductionVector_base(LargePhyto_rowclm)     	= P_LargePhyto_base;

InputProductionVector_scenario(SmlPhyto_rowclm)    	= P_SmlPhyto_base;      % use base for now, can change these for a scenario
InputProductionVector_scenario(LargePhyto_rowclm)  	= P_LargePhyto_base;    % use base for now, can change these for a scenario


% step 7b: evaluate type of driver (nutrient vs living group) -------------
% NOTE: THIS IS NEEDED IF DRIVEN BY ANYTHING OTHER THAN NO3
TestInput_base                      = InputProductionVector_base;
TestInput_scenario                  = InputProductionVector_scenario;
TestInput_base(looky_NUTRIENTS)     = [];
TestInput_scenario(looky_NUTRIENTS) = [];
if max(TestInput_base > 0 | TestInput_scenario > 0); 
    disp('WARNING: non-NO3 driver. Be sure to deactivate nutrient recycling; set Nutrient columns in production matrix to 0');
    EnergyBudget_matrix(:, looky_NUTRIENTS, :)    = 0; % shut off recycling
    % NOTE: this automatic switch assumes that both the base and the scenario are using non-nutrient drivers
    % Will need to add a more sophisticated switch to allow a nutrient driver base and a non-nutrient driver scenario (or vice-versa)
    % FFF in future, can have nutrient and non-nutrient drives for different subregions
else
    disp('WARNING: NO3 driver. Be sure to keep nutrient recycling ACTIVATED'); 
end





% *************************************************************************
% STEP 8: calculate base productivity--------------------------------------
for MonteCarlo_loop = 1:num_webMonteCarlo    
    current_EnergyBudget_matrix         = EnergyBudget_matrix(:, :, MonteCarlo_loop);
    current_TransferEfficiency          = TransferEfficiency(MonteCarlo_loop, :);
    Production_base(:, MonteCarlo_loop) = f_WebProductivityWLoss(current_TransferEfficiency, current_EnergyBudget_matrix, InputProductionVector_base, PhysicalLossFraction_base); % (2D matrix: E2E_clms X num_webMonteCarlo)
end % (MonteCarlo_loop)





% *************************************************************************
% STEP 9: SCENARIOS--------------------------------------------------------

% example 1) MAMMAL scenario: baleen whales scaled up X5 (p = ALL) ----------------
ScenarioConditions.trgt_producer           = 1:E2E_clms;                    % column number(s) of producer group(s) that are subject to consumption
ScenarioConditions.modify_consumer         = [BalWhal_rowclm];
ScenarioConditions.ScaleFactor             = 5;                             % value > 1 means increase flow to modify_consumer (and reduced flow to offset_consumer); 
                                                                            % value < 1 means decrease flow to modify_consumer (and increased flow to offset_consumer); 
                                                                            % NOTE: keep ScaleFactor >= 0 (negative value makes no sense)
ScenarioConditions.offset_consumer         = [1:E2E_clms];                  % indentify consumers that directly gain or lose due to forced changes to modify_consumer 
ScenarioConditions.offset_consumer([looky_NUTRIENTS; looky_DETRITUS]) = []; % (assume no change to detritus or to nutrients; --YOU ARE FREE TO CHANGE THIS ASSUMPTION--!!!)
ScenarioConditions.EnergyBudget_base       = EnergyBudget_matrix;           % this is MonteCarlo set of EnergyBudget_matrix models
ScenarioConditions.TransferEfficiency      = TransferEfficiency(1, :);      % choose the first TransferEfficiency (there's 1 row for all MonteCarlo models but they are all the same)
ScenarioConditions.InputProductionVector   = InputProductionVector_base;    % <<<---NOTE base input production driver rates
ScenarioConditions.PhysicalLossFraction    = PhysicalLossFraction_base;     % <<<---NOTE base advection rates
ScenarioConditions.force_offset            = 'n'; % ('y' means to force offset consumer fractions evenly, even if original value is 0; 'n' means to distribute proportionally to original values

[EnergyBudget_scenario Production_scenario BuildGroup max_ScaleFactor] = f_ScenarioGenerator_08302013(ScenarioConditions, ECOTRAN); % modify the EnergyBudget_matrix according to forced scenario conditions


% step 9b: compile scenario results (scenario_FractionalChange = change relative to un-altered model; scenario_RatioChange = altered to un-altered model ratio)
[FractionalChange, RatioChange, scenario_ResultsCompiled] = f_CompileScenarioResults_08192013(Production_base, Production_scenario, max_ScaleFactor);
% scenario_ResultsCompiled:
%       clm 1: FractionalChange_mean
%       clm 2: FractionalChange_std
%       clm 3: RatioChange_mean 
%       clm 4: RatioChange_std
%       clm 5: Production_scenario_mean 
%       clm 6: Production_scenario_std 
%       clm 7: Production_base_mean 
%       clm 8: Production_base_std

% step 9c: plot scenario results ------------------------------------------
%       scenario_FractionalChange: change relative to un-altered model
%       scenario_RatioChange: altered to un-altered model ratio)
%       choose which to plot within code p_PlotScenarioResults
target_groups = [PlankFish_rowclm; PiscFish_rowclm; BenthFish_rowclm; Cephalo_rowclm; ...
    SeaBird_rowclm; BalWhal_rowclm; Pinniped_rowclm; Odontocete_rowclm; ...
    CarnBen_rowclm; Fishery1_rowclm; Fishery2_rowclm]; % select groups to plot along x-axis; set to empty ([]) to plot all

y_axis        = [-1 1]; % define y-axis in plots
p_PlotScenarioResults_02092018(FractionalChange, RatioChange, AggLabel, BuildGroup, target_groups, y_axis)


% end m-file***************************************************************