function [DietMatrix_NoCannibalism, cannibalism_vector] = f_RedistributeCannibalism(DietMatrix)
% STEP 2: get rid of the non-zero diag terms; redistribute cannibalism
% terms among all other non-zero terms for each predator (columns)
% takes:
%   DietMatrix
% returns:
%   DietMatrix_NoCannibalism        (matrix)
%   cannibalism_vector              (horizontal vector)
% revision date: 5-10-2010
% corrected error when re-scaling each column to 1 (5-10-10)

% STEP 1: measure size of diet matrix
[rows, clms] = size(DietMatrix);
if rows ~= clms
    error('diet matrix must be square')
end

% STEP 2: retain cannibalism vector
cannibalism_vector = diag(DietMatrix)';

% STEP 3: set cannibalism diagonal to 0 in DietMatrix_NoCannibalism
DietMatrix_NoCannibalism = DietMatrix;
for row_loop = 1:rows
    DietMatrix_NoCannibalism(row_loop, row_loop) = 0;
end

% STEP 4: rescale each clm in DietMatrix_NoCannibalism to 1
for clm_loop = 1:clms
    DietMatrix_NoCannibalism(:, clm_loop) = DietMatrix_NoCannibalism(:, clm_loop) ./ sum(DietMatrix_NoCannibalism(:, clm_loop));
end
DietMatrix_NoCannibalism(isnan(DietMatrix_NoCannibalism)) = 0; % convert NaNs (phyto & detritus groups) to 0

% end m-file***************************************************